/* Copyright (c) 2012 Boulder Real Time Technologies, Inc. */
/* All rights reserved */
 
/* This software module is wholly owned by Boulder Real Time 
   Technologies, Inc. This software may be used freely in any 
   way as long as the copyright statement above is not removed. */

#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>

#include "orbwfproc.h"
#include "tr.h"
#include "sysdata.h"

static void
orb2wf_skin_usage (void)

{
	fprintf (stderr, "usage: orb2wf [-c chanmatch] [-m srcmatch] [-r srcreject]\n");
	fprintf (stderr, "              [-p pf-file] [-S state-file] [{-v|-vv|-vvv}]\n");
	fprintf (stderr, "              [-dbm dbmaster] orb db [start-time [window]]\n");
}

#include "orbwfproc_version.h"

static void
startup_banner(void)

{
	int i, n;
	char *pre1 = "******";
	char *pre2 = "*     ";
	char *post1 = "******";
	char *post2 = "     *";
	char line1[256], line2[256];

	n = strlen(Orbwfproc_Version);
	strcpy (line1, "*");
	for (i=1; i<n; i++) {
		strcat (line1, "*");
	}

	elog_notify (0, "\n");
	elog_notify (0, "%s%s%s\n", pre1, line1, post1);
	strcpy (line2, " ");
	for (i=1; i<n; i++) {
		strcat (line2, " ");
	}
	memcpy (line2, "orb2wf startup", strlen("orb2wf startup"));
	elog_notify (0, "%s%s%s\n", pre2, line2, post2);
	elog_notify (0, "%s%s%s\n", pre2, Orbwfproc_Version, post2);
	elog_notify (0, "%s%s%s\n", pre1, line1, post1);

}

int
orb2wf_skin_setup (int argc, char **argv, char **pfcmd_str, Pf **pf)

{
	char *dbmaster=NULL;
	void *vptr;
	double pause_timeout=1800.0;
	WFProcDb *dbm;
	char line[256];
	void *pfvptr;
	char *ptr;
	int calibfromdb=0;
	int process_through_unstuff_errors=0;
	int add_wfdisc_headers=1;
	char *chanmatch = NULL;
	char pfname[256];
	Tbl *tbl;
	int i;

	vptr = NULL;

	if (argc < 2) {
		orb2wf_skin_usage();
		banner ("orb2wf", 0) ;
		exit (1);
	}
	startup_banner();
	strcpy (pfname, "orb2wf");

	for (argv++,argc--; argc>0; argv++,argc--) {
		if (**argv != '-') break;
		if (!strcmp(*argv, "-c")) {
			argc--; argv++;
			if (argc < 1) {
				complain (0, "Need -c argument.\n");
				orb2wf_skin_usage();
				exit (1);
			}
			chanmatch = *argv;
		} else if (!strcmp(*argv, "-m")) {
			argc--; argv++;
			if (argc < 1) {
				complain (0, "Need -m argument.\n");
				orb2wf_skin_usage();
				exit (1);
			}
			pushstr (&vptr, "-select ");
			pushstr (&vptr, *argv);
			pushstr (&vptr, "\n");
		} else if (!strcmp(*argv, "-r")) {
			argc--; argv++;
			if (argc < 1) {
				complain (0, "Need -r argument.\n");
				orb2wf_skin_usage();
				exit (1);
			}
			pushstr (&vptr, "-reject ");
			pushstr (&vptr, *argv);
			pushstr (&vptr, "\n");
		} else if (!strcmp(*argv, "-S")) {
			argc--; argv++;
			if (argc < 1) {
				complain (0, "Need -S argument.\n");
				orb2wf_skin_usage();
				exit (1);
			}
			pushstr (&vptr, "-state ");
			pushstr (&vptr, *argv);
			pushstr (&vptr, "\n");
		} else if (!strcmp(*argv, "-p")) {
			argc--; argv++;
			if (argc < 1) {
				complain (0, "Need -p argument.\n");
				orb2wf_skin_usage();
				exit (1);
			}
			strcpy (pfname, *argv);
		} else if (!strcmp(*argv, "-dbm")) {
			argc--; argv++;
			if (argc < 1) {
				complain (0, "Need -dbm argument.\n");
				orb2wf_skin_usage();
				exit (1);
			}
			dbmaster = *argv;
		} else if (!strcmp(*argv, "-v")) {
			pushstr (&vptr, "-verbose 1\n");
		} else if (!strcmp(*argv, "-vv")) {
			pushstr (&vptr, "-verbose 2\n");
		} else if (!strcmp(*argv, "-vvv")) {
			pushstr (&vptr, "-verbose 3\n");
		} else {
			complain (0, "Unrecognized argument '%s'.\n", *argv);
			orb2wf_skin_usage();
			exit (1);
		}
	}
	
	if (argc < 1) {
		complain (0, "Need orb argument.\n");
		orb2wf_skin_usage();
		exit (1);
	}
	pushstr (&vptr, "orbin ");
	pushstr (&vptr, *argv);
	pushstr (&vptr, "\n");

	argv++; argc--;

	if (argc < 1) {
		complain (0, "Need db argument.\n");
		orb2wf_skin_usage();
		exit (1);
	}
	pushstr (&vptr, "dbout ");
	pushstr (&vptr, *argv);
	pushstr (&vptr, "\n");

	argv++; argc--;

	if (argc > 0) {
		double tstart;

		if (zstr2epoch (*argv, &tstart) < 0) {
			complain (0, "Unable to parse start-time argument, '%s'.\n", *argv);
			orb2wf_skin_usage();
			exit (1);
		}
		pushstr (&vptr, "-start ");
		pushstr (&vptr, *argv);
		pushstr (&vptr, "\n");
		argv++; argc--;
		if (argc > 0) {
			double twin, tend;

			if (zstr2epoch (*argv, &twin) < 0) {
				complain (0, "Unable to parse window argument, '%s'.\n", *argv);
				orb2wf_skin_usage();
				exit (1);
			}
			tend = tstart + twin;

			pushstr (&vptr, "-end ");
			sprintf (line, "%lf", tend);
			pushstr (&vptr, line);
			pushstr (&vptr, "\n");
			argv++; argc--;
			if (argc > 0) {
				complain (0, "Unrecognized argument '%s'.\n", *argv);
				orb2wf_skin_usage();
				exit (1);
			}
		}
	}

	/* Read orb2wf parameter file */

	if (pfread (pfname, pf) < 0) {
		die (0, "pfread(%s) error.\n", pfname);
	}

	if (parse_param (*pf, "calibfromdb", P_BOOL, 0, &calibfromdb) < 0) {
		die (0, "parse_param(calibfromdb) error.\n");
	}
	if (parse_param (*pf, "process_through_unstuff_errors", P_BOOL, 0, &process_through_unstuff_errors) < 0) {
		die (0, "parse_param(process_through_unstuff_errors) error.\n");
	}
	if (calibfromdb && !dbmaster) {
		die (0, "calibfromdb in parameter file set to yes but no -dbm database specified.\n");
	}
	if (parse_param (*pf, "pause_timeout", P_DBL, 1, &pause_timeout) < 0) {
		die (0, "parse_param(pause_timeout) error.\n");
	}
	sprintf (line, "-pause_timeout %lf\n", pause_timeout);
	pushstr (&vptr, line);
	ptr = NULL;
	if (parse_param (*pf, "decent_interval", P_STR, 1, &ptr) < 0) {
		die (0, "parse_param(decent_interval) error.\n");
	}
	sprintf (line, "-statefile_update_interval %s\n", ptr);
	pushstr (&vptr, line);
	free (ptr);

	sprintf (line, "-pause_timeout %lf\n", pause_timeout);
	pushstr (&vptr, line);

	pfvptr = NULL;
	pushstr (&pfvptr, "wf_tasks &Tbl{\n");
	pushstr (&pfvptr, "import import import_params - wf\n");
	pushstr (&pfvptr, "export export export_params wf -\n");
	pushstr (&pfvptr, "}\n");
	pushstr (&pfvptr, "import_params &Arr{\n");
	pushstr (&pfvptr, "input_type orb\n");
	pushstr (&pfvptr, "input orbin\n");
	sprintf (line, "calibfromdb %d\n", calibfromdb);
	pushstr (&pfvptr, line);
	sprintf (line, "process_through_unstuff_errors %d\n", process_through_unstuff_errors);
	pushstr (&pfvptr, line);
	if (chanmatch) {
		pushstr (&pfvptr, "chan_accept &Tbl{\n");
		pushstr (&pfvptr, chanmatch);
		pushstr (&pfvptr, "}\n");
	}
	ptr = NULL;
	if (parse_param (*pf, "heap_throttle", P_STR, 0, &ptr) < 0) {
		die (0, "parse_param(heap_throttle) error.\n");
	}
	if (ptr) {
		sprintf (line, "heap_throttle %s\n", ptr);
		pushstr (&pfvptr, line);
		free (ptr);
	}
	ptr = NULL;
	if (parse_param (*pf, "too_old", P_STR, 1, &ptr) < 0) {
		die (0, "parse_param(too_old) error.\n");
	}
	sprintf (line, "too_old %s\n", ptr);
	pushstr (&pfvptr, line);
	free (ptr);
	ptr = NULL;
	if (parse_param (*pf, "too_new", P_STR, 1, &ptr) < 0) {
		die (0, "parse_param(too_new) error.\n");
	}
	sprintf (line, "too_new %s\n", ptr);
	pushstr (&pfvptr, line);
	free (ptr);
	pushstr (&pfvptr, "}\n");
	pushstr (&pfvptr, "export_params &Arr{\n");
	pushstr (&pfvptr, "output_type db\n");
	pushstr (&pfvptr, "output dbout\n");
	pushstr (&pfvptr, "fragment_wfdisc no\n");
	pushstr (&pfvptr, "addcheck no\n");
	ptr = NULL;
	if (parse_param (*pf, "preferred_waveform_file_range", P_STR, 1, &ptr) < 0) {
		die (0, "parse_param(preferred_waveform_file_range) error.\n");
	}
	sprintf (line, "wfduration %s\n", ptr);
	pushstr (&pfvptr, line);
	free (ptr);
	ptr = NULL;
	if (parse_param (*pf, "max_open_files", P_STR, 1, &ptr) < 0) {
		die (0, "parse_param(max_open_files) error.\n");
	}
	sprintf (line, "max_open_files %s\n", ptr);
	pushstr (&pfvptr, line);
	free (ptr);
	ptr = NULL;
	if (parse_param (*pf, "maxduration", P_STR, 1, &ptr) < 0) {
		die (0, "parse_param(maxduration) error.\n");
	}
	sprintf (line, "maxduration %s\n", ptr);
	pushstr (&pfvptr, line);
	free (ptr);
	ptr = NULL;
	if (parse_param (*pf, "samprate_thresh", P_STR, 1, &ptr) < 0) {
		die (0, "parse_param(samprate_thresh) error.\n");
	}
	sprintf (line, "samprate_thresh %s\n", ptr);
	pushstr (&pfvptr, line);
	free (ptr);
	ptr = NULL;
	if (parse_param (*pf, "calib_thresh", P_STR, 1, &ptr) < 0) {
		die (0, "parse_param(calib_thresh) error.\n");
	}
	sprintf (line, "calib_thresh %s\n", ptr);
	pushstr (&pfvptr, line);
	free (ptr);
	ptr = NULL;
	if (parse_param (*pf, "calper_thresh", P_STR, 1, &ptr) < 0) {
		die (0, "parse_param(calper_thresh) error.\n");
	}
	sprintf (line, "calper_thresh %s\n", ptr);
	pushstr (&pfvptr, line);
	free (ptr);
	ptr = NULL;
	if (parse_param (*pf, "nullcalib", P_STR, 1, &ptr) < 0) {
		die (0, "parse_param(nullcalib) error.\n");
	}
	sprintf (line, "nullcalib %s\n", ptr);
	pushstr (&pfvptr, line);
	free (ptr);
	ptr = NULL;
	if (parse_param (*pf, "realtime_thresh", P_STR, 1, &ptr) < 0) {
		die (0, "parse_param(realtime_thresh) error.\n");
	}
	sprintf (line, "realtime_thresh %s\n", ptr);
	pushstr (&pfvptr, line);
	free (ptr);
	ptr = NULL;
	if (parse_param (*pf, "datatype", P_STR, 1, &ptr) < 0) {
		die (0, "parse_param(datatype) error.\n");
	}
	sprintf (line, "datatype %s\n", ptr);
	pushstr (&pfvptr, line);
	free (ptr);
	ptr = NULL;
	if (parse_param (*pf, "gain", P_STR, 0, &ptr) < 0) {
		die (0, "parse_param(gain) error.\n");
	}
	if (ptr) {
		sprintf (line, "gain %s\n", ptr);
		pushstr (&pfvptr, line);
		free (ptr);
	}
	if (parse_param (*pf, "add_wfdisc_headers", P_BOOL, 0, &add_wfdisc_headers) < 0) {
		die (0, "parse_param(add_wfdisc_headers) error.\n");
	}
	sprintf (line, "add_wfdisc_headers %d\n", add_wfdisc_headers);
	pushstr (&pfvptr, line);
	tbl = NULL;
	if (parse_param (*pf, "translations", P_TBL, 0, &tbl) < 0) {
		die (0, "parse_param(translations) error.\n");
	}
	if (tbl) {
		pushstr (&pfvptr, "translations &Tbl{\n");
		for (i=0; i<maxtbl(tbl); i++) {
			char *entry;

			entry = (char *) gettbl (tbl, i);
			pushstr (&pfvptr, entry);
			pushstr (&pfvptr, "\n");
		}
		pushstr (&pfvptr, "}\n");
	}
	pushstr (&pfvptr, "}\n");
	ptr = popstr (&pfvptr, 1);
	if (pfcompile (ptr, pf) < 0) {
		die (0, "pfcompile(wf_tasks table) error.\n");
	}
	free (ptr);

	if (dbmaster) {
		dbm = wfprocdb_create (dbmaster, NULL, 1, pause_timeout);
		if (dbm == NULL) {
			die (0, "wfprocdb_create(%s) error.\n", dbmaster);
		}

		sprintf (line, "dbmaster %lu\n", (unsigned long) dbm);
		pushstr (&vptr, line);
	}

	/* Merge command line args with parameter file */

	*pfcmd_str = popstr (&vptr, 1);

	if (pfcompile (*pfcmd_str, pf) < 0) {
		die (0, "pfcompile(pftask) error.\n");
	}

	return (0);
}
